use derive_more::Constructor;
use diesel::{AsChangeset, Identifiable, Insertable, Queryable, Selectable};
use lib::diesel_crud_derive::DieselCrud;
use lib::into_response_derive::IntoResponse;
use serde::{Deserialize, Serialize};
use validator::Validate;

#[derive(
    Queryable, Selectable, Identifiable, AsChangeset, Serialize, IntoResponse, DieselCrud, Validate,
)]
#[diesel_crud(insert = CreateHotel)]
#[diesel(table_name = crate::schema::hotel)]
#[diesel(check_for_backend(diesel::pg::Pg))]
pub struct Hotel {
    #[diesel_crud(pk)]
    pub id: i32,
    #[validate(length(min = 1))]
    pub name: String,
    #[validate(length(min = 1))]
    pub address: String,
}

#[derive(Insertable, Deserialize, Validate, Constructor)]
#[diesel(table_name = crate::schema::hotel)]
#[diesel(check_for_backend(diesel::pg::Pg))]
pub struct CreateHotel {
    #[validate(range(min = 1))]
    pub id: Option<i32>,
    #[validate(length(min = 1))]
    pub name: String,
    #[validate(length(min = 1))]
    pub address: String,
}
