use diesel::{AsChangeset, Associations, Identifiable, Insertable, Queryable, Selectable};
use lib::diesel_crud_derive::DieselCrud;
use lib::into_response_derive::IntoResponse;
use serde::{Deserialize, Serialize};
use validator::Validate;

#[derive(
    Debug,
    PartialEq,
    Queryable,
    Selectable,
    Identifiable,
    Associations,
    AsChangeset,
    Insertable,
    Serialize,
    Deserialize,
    IntoResponse,
    Validate,
    DieselCrud,
)]
#[diesel(belongs_to(crate::models::hotel::Hotel))]
#[diesel(table_name = crate::schema::room)]
#[diesel(check_for_backend(diesel::pg::Pg))]
pub struct Room {
    #[diesel_crud(pk)]
    pub id: i32,
    #[validate(range(min = 1))]
    pub hotel_id: i32,
    #[validate(range(min = 1))]
    pub beds: i32,
    #[validate(range(min = 1))]
    pub size: i32,
}
