use axum::extract::{Path, State};

use lib::diesel_crud_trait::{DieselCrudList, DieselCrudRead};

use crate::auth::{authenticate, AuthSession};
use crate::models::hotel::Hotel;
use crate::models::user::UserRole;
use crate::result::{ResponseResult, Success};
use crate::GetConnection;
use lib::axum::wrappers::Array;
use lib::{router, routes};

router!(
    "/hotel",
    routes! {
        get "/:id" => get_hotel::<T>,
        get "/list" => list_hotels::<T>,
    },
    T: Clone, GetConnection
);

async fn get_hotel<Pool>(
    auth_session: AuthSession<Pool>,
    State(pool): State<Pool>,
    Path(id): Path<i32>,
) -> ResponseResult<Hotel>
where
    Pool: GetConnection,
{
    authenticate(auth_session.user.as_ref(), [UserRole::Admin], |_| async {
        Hotel::read(id, pool.get().await?.as_mut())
            .await
            .map(Success::Ok)
            .map_err(Into::into)
    })
    .await
}

async fn list_hotels<Pool>(
    auth_session: AuthSession<Pool>,
    State(pool): State<Pool>,
) -> ResponseResult<Array<Hotel>>
where
    Pool: GetConnection,
{
    authenticate(auth_session.user.as_ref(), [UserRole::Admin], |_| async {
        Hotel::list(pool.get().await?.as_mut())
            .await
            .map(Array::from)
            .map(Success::Ok)
            .map_err(Into::into)
    })
    .await
}
