use crate::auth::{authenticate, AuthSession};
use crate::models::user::{User, UserRole};
use crate::result::{ResponseResult, Success};
use crate::GetConnection;
use axum::extract::{Path, State};
use lib::axum::wrappers::Array;
use lib::diesel_crud_trait::{DieselCrudList, DieselCrudRead};
use lib::{router, routes};

router!(
    "/user",
    routes!(
        get "/:email" => get_user::<T>,
        get "/" => list_users::<T>
    ),
    T: Clone, GetConnection
);
async fn get_user<Pool>(
    auth_session: AuthSession<Pool>,
    Path(email): Path<String>,
    State(pool): State<Pool>,
) -> ResponseResult<User>
where
    Pool: GetConnection,
{
    authenticate(auth_session.user.as_ref(), [UserRole::Admin], |_| async {
        User::read(email, pool.get().await?.as_mut())
            .await
            .map(Success::Ok)
            .map_err(Into::into)
    })
    .await
}

async fn list_users<Pool>(
    auth_session: AuthSession<Pool>,
    State(pool): State<Pool>,
) -> ResponseResult<Array<User>>
where
    Pool: GetConnection,
{
    authenticate(auth_session.user.as_ref(), [UserRole::Admin], |_| async {
        User::list(pool.get().await?.as_mut())
            .await
            .map(Array::from)
            .map(Success::Ok)
            .map_err(Into::into)
    })
    .await
}
